/********************************************************************\
 * duifield-gobj.c -- Implementation of the glib-2.0 GObject field  *
 * Copyright (C) 2003 Linas Vepstas <linas@linas.org>               *
 * http://dwi.sourceforge.net                                       *
 *                                                                  *
 * This library is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU Lesser General Public       *
 * License as published by the Free Software Foundation; either     *
 * version 2.1 of the License, or (at your option) any later version.
 *                                                                  *
 * This library is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU Lesser General Public License for more details.              *
 *                                                                  *
 * You should have received a copy of the GNU Lesser General Public *
 * License along with this program; if not, contact:                *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
\********************************************************************/

/*
 * FUNCTION:
 * Implementation of the glib-2.0 GObject field 
 *
 * HISTORY:
 * Linas Vepstas September 2003
 */

#include "config.h"

#include <glib.h>
#include <string.h>

#include "duifield.h"
#include "duifield-gobj.h"
#include "duiresolver.h"
#include "interface.h"
#include "perr.h"

/* A property on a glib-2.0 gobject */
struct gobj_field_s
{
	DuiInterface *gobj_root;
	char * property;
};

/* =================================================================== */

#define GOBJ(f) ((struct gobj_field_s *)(&((f)->u.priv)))

static void
gobj_field_clear (DuiField *f)
{
	g_free (f->fieldname);
	g_free (GOBJ(f)->property);
	f->type = DUI_FIELD_NONE;
}

/* =================================================================== */

static void 
set_gobj_value (DuiField *fs, const char *value)
{
	GValue val = {0};
	GObject *gob;

	gob = dui_interface_find_object_by_name(GOBJ(fs)->gobj_root, 
	                fs->fieldname);
	if (!gob)
	{
		PERR ("Can't find object named \'%s\'\n", fs->fieldname);
		return;
	}
									
	g_value_init (&val, G_TYPE_STRING);
	g_value_set_string (&val, value);
	g_object_set_property (gob, GOBJ(fs)->property, &val);
}

void
dui_field_set_gobj (DuiField *ft, const char * obj,
                                  const char * prop)
{
	struct gobj_field_s *gf;

	ft->type = DUI_FIELD_GOBJ;
	ft->fieldname = g_strdup (obj);

	ft->get_field_value = NULL;  /* XXX impmenet this */
	ft->set_field_value = set_gobj_value;
	ft->clear_field = gobj_field_clear;

	gf = GOBJ(ft);
	gf->property = g_strdup (prop);
}

/* =================================================================== */

void 
dui_resolver_resolve_gobj (DuiResolver *res , DuiInterface *rot)
{
	GList *node;

	for (node = res->field_list; node; node=node->next)
	{
		DuiField *fld = node->data;
		if (!DUI_FIELD_IS_TYPE(fld,DUI_FIELD_GOBJ)) continue;

		GOBJ(fld)->gobj_root = rot;
	}
}

/* ========================== END OF FILE ============================ */
