/********************************************************************\
 * duifield.h - abstract base class for a 'field'.                  *
 * Copyright (C) 2003 Linas Vepstas <linas@linas.org>               *
 * http://dwi.sourceforge.net                                       *
 *                                                                  *
 * This library is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU Lesser General Public       *
 * License as published by the Free Software Foundation; either     *
 * version 2.1 of the License, or (at your option) any later version.
 *                                                                  *
 * This library is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU Lesser General Public License for more details.              *
 *                                                                  *
 * You should have received a copy of the GNU Lesser General Public *
 * License along with this program; if not, contact:                *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
\********************************************************************/

/*
 * FUNCTION:
 * Abstract base class for the 'field'.
 * The 'field maps to a column on an sql db, or to the value 
 * contained in an interface widget, etc. etc. 
 *
 * HISTORY:
 * Copyright (c) 2003 Linas Vepstas <linas@linas.org>
 */

#ifndef DUI_FIELD_H
#define DUI_FIELD_H

#include "config.h"
#include <string.h>

typedef struct DuiField_s DuiField;
typedef const char * DuiFieldType; 

#define DUI_FIELD_NONE      "DUI_FIELD_NONE"
#define DUI_FIELD_CONST     "DUI_FIELD_CONST"    /* Field with const value */
#define DUI_FIELD_WIDGET    "DUI_FIELD_WIDGET"   /* Value in gtk widget itself */
#define DUI_FIELD_WID_DATA  "DUI_FIELD_WID_DATA"  /* Value from gtk-1.2 widget data */
#define DUI_FIELD_WID_ARG   "DUI_FIELD_WID_ARG"   /* Value from gtk-1.2 widget GtkArg */
#define DUI_FIELD_WID_WHERE "DUI_FIELD_WID_WHERE" /* Match to a particular gtk widget */
#define DUI_FIELD_HASH_KEY  "DUI_FIELD_HASH_KEY"  /* Global hash table key */
#define DUI_FIELD_SQL       "DUI_FIELD_SQL"       /* Database table column */
#define DUI_FIELD_WHERE     "DUI_FIELD_WHERE"     /* match to database column */
#define DUI_FIELD_GOBJ      "DUI_FIELD_GOBJ"      /* glib-2.0 gobject */

#define DUI_FIELD_IS_TYPE(fld,typ) (0==strcmp((fld)->type,typ))

struct DuiField_s
{
	DuiFieldType type;
	char * fieldname;
	const char * (*get_field_value) (DuiField *);
	void (*set_field_value) (DuiField *, const char *);
	void (*clear_field) (DuiField *);
	union 
	{
		/* just a plain static, const value */
		char * value;

		/* private, per-type data area */
		gpointer priv[8];
	} u;
};

/** Get the name of the field */
const char * dui_field_get_fieldname (DuiField *fs);

void dui_field_clear (DuiField *s);

/** data will be a constant unchanging value */
void dui_field_set_const (DuiField *fs, const char *value);

#endif /* DUI_FIELD_H */
